#!/usr/bin/env python
##------------------------------------------------------------------------------
##
## ac_xxx.py
##
## This is a rewrite of the csh script using the common elements developed when 
## writing a script to create copies of certain COS files.
##
## AC, standing for After Calibration, may do different things for different
## instrument data, or may do the same thing for every instrument.
## For example, for COS we will copy some files so catalog will work.
## For NIC, we want to ...
##
## Input:
##
## History:
## mm/dd/yy PR    Who        What          
## -------- ----- ---------- --------------------------------------------------
## 12/29/08 60896 Sherbert   change from csh to py
## 01/21/09 61586 Sherbert   Add celestial coord update for NIC prods
## 03/10/09 62166 Sherbert   Create cos file copies elsewhere
##
##------------------------------------------------------------------------------

from ac_pkg import  *
import create_fake_files_for_cos_cat
import export_saa_dark
import quality_update
import celestial_coord_update
import opusutil
from opusutil_pyfits import update_coord_keywords

moduleName = os.path.basename(sys.argv[0])

# ## exit variables
# ALL_IS_WELL = 0
# GO_ABSENT = 102          #this exit status is in the FATAL range for XPOLL
# QUIT_THIS_DATASET = 1    #this exit status is mapped and will allow
# 
# 
# true  = 1
# false = 0

##------------------------------------------------------------------------------
##
## Method:  usage()
##
## Description: 
##          Print usage message for ac_xxx(.py)
##
## Returns: nothing
##
##------------------------------------------------------------------------------
def usage():
    usageMsg = """
            ac_xxx.py 

            If no command-line arguments are provided, environment variables 
            must be defined for the 7 arguments.  Otherwise, the arguments are 
            expected to be in this order:

            <KW_SERVER> <KW_DB> <ARCH_SERVER> <ARCH_DB> 
                                <SAADARK_CACHE> <OSF_DATASET> <OUTPATH>

            <ARCH_SERVER>, <ARCH_DB> are the DADS/archive server and database 
            <KW_SERVER>, <KW_DB> are the keyword server and database 
            <SAADARK_CACHE> is the dir to contain SAA dark files
            <OSF_DATASET> is expected to exist in <OUTPATH>.  
            
            This script will 
            1) run asn_product_status.pl for all instruments 
               unless we replace that with something better seeing as 
               how many of our tasks require reading the ASN table file...
            2) export NIC SAA Dark files to the cache area (export_saa_dark.py)
            3) perform PDQ quality keyword updates 
            4) create fake COS files for cataloging (create_fake_files_for_cos_cat.py)

            

    """
#   opusutil.PrintMsg('F', usageMsg, moduleName)
    print usageMsg
    return

##------------------------------------------------------------------------------
##
## Main
##
##    Nothing intended to be called from anything else, so no need to
##    check for __main__, right?
## 
## History:
## mm/dd/yy PR    Who        What          
## -------- ----- ---------- --------------------------------------------------
## 08/12/09 63318 Sherbert   catch an exception
## 06/01/10 64909 Sherbert   QUAL* updates for non-members except WF2, STIS
##
##------------------------------------------------------------------------------
## Want this script to be runnable from command line or from pipeline
banner()
envVars = {}

needThese = ["KW_SERVER", "KW_DB", "ARCH_SERVER", "ARCH_DB", "OPUS_SERVER", "OPUS_DB", "SAADARK_CACHE", "OSF_DATASET", "OUTPATH"]
if (len(sys.argv)) == 1:
    opusutil.PrintMsg('D', 'Processing env vars', moduleName )
    try:
        envVars = envSetUp('env', needThese)
    except RuntimeError, errMsg:
        opusutil.PrintMsg('F', str(errMsg), moduleName)
        usage()
        sys.exit(GO_ABSENT)
else:
    opusutil.PrintMsg('D', 'Processing cmd line vars', moduleName )
    try:
        envVars = envSetUp('cmdLine', needThese) 
    except AttributeError, errMsg:
        opusutil.PrintMsg('F', str(errMsg), moduleName)
        usage()
        sys.exit(GO_ABSENT)

KW_SERVER   = envVars["KW_SERVER"] 
KW_DB       = envVars["KW_DB"] 
ARCH_SERVER = envVars["ARCH_SERVER"] 
ARCH_DB     = envVars["ARCH_DB"] 
cache       = envVars["SAADARK_CACHE"]
rootname    = envVars["OSF_DATASET"]
caldir      = envVars["OUTPATH"]

## Verify and cd to caldir
try:
    verify_n_cd_to_dir(caldir)
except RuntimeError, errMsg:
    opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.verify_n_cd_to_dir')
    sys.exit(GO_ABSENT)

## Can someone tell me why WF2 trailers are still trl?
trl_ext = '.tra' 
if rootname[0] == 'u':
    trl_ext = '.trl' 
## Unix filenames pretty reliably lower-cased
trailer = rootname.lower() + trl_ext
if not os.path.exists(trailer):
    msg = "Creating new " + trailer
    opusutil.PrintMsg("W", msg, moduleName)
try:
    opusutil.OpenTrl(trailer)
except:
    msg = "Cannot open " + trailer + ".  Messages to process log only."
    opusutil.PrintMsg("W", msg, moduleName)

ID   = '-------------- After Calibration '
strt = 'Started for   '
endg = 'Completed for '
ID2  = rootname + ' --------------'
strtMsg = ID + strt + ID2
endgMsg = ID + endg + ID2
opusutil.PrintMsg('I', strtMsg )

## Verify we are dealing with NIC data before calling
## export_saa_dark.py
if verify_SI(rootname, "NICMOS"):
    msg = "Calling export_saa_dark for NIC "
    opusutil.PrintMsg('D', msg, moduleName )
    stat = export_saa_dark.doIt(envVars)
    if stat == GO_ABSENT:
        banner()
        sys.exit(GO_ABSENT)
    elif stat > 0:
        ## SAADARK_CACHE population NOT supposed to be important 
        ## enough to stop processing.  Write error message w/o exit.
        msg  = "Too many errors! (" + str(stat)  + ") "
        msg += "SAADARK copies NOT all successful. "
        opusutil.PrintMsg('E', msg, moduleName )

    ## Which files to update?  
    ## The NIC files that have ASN_MTYP==PROD-TARG AND contain
    ## the [GE]{LAT|LON}_REF keywords are the mos and saa files.
    ## The team may not like the saa files, but we DO catalog from 
    ## them, so I am keeping them here.
    ## Do I want to hard-code those extensions, or read every NIC file?
    ## I fear opening every NIC header is going to slow this 
    ## process down a bit on our system.  Therefore hard-coded for now.
    msg = "Calling update_celestial_pos for NIC PROD-TARG"
    opusutil.PrintMsg('D', msg, moduleName )
    stat = celestial_coord_update.doIt( envVars )
    ## How do I want to handle an error?
    if stat > 0:
        msg  = "Celestial coordinate keyword update NOT successful. "
        opusutil.PrintMsg('E', msg, moduleName )
        sys.exit(QUIT_THIS_DATASET)


msg = "Calling quality_update for all SIs except STIS, WF2 "
opusutil.PrintMsg('D', msg, moduleName )
stiObs = verify_SI(rootname, "STIS") 
wf2Obs = verify_SI(rootname, "WF2") 
if not stiObs and not wf2Obs :
    stat = quality_update.doIt(envVars) 
    if ( stat != 0 ):
        sys.exit(stat)


msg = "Calling asn_product_status.pl for all SIs "
opusutil.PrintMsg('D', msg, moduleName )

## I would consider rewriting this script under a different PR 
## but we don't think it would get very high priority, so call 
## perl script from python.
cmd = 'asn_product_status.pl ' + rootname
apsOut = opusutil.getListOutputOf( cmd )
for line in apsOut:
    ## Here's a dilemma:
    ## line already has opusutil.PrintMsg form, 
    ## Therefore I would rather use print
    ## but it is opusutil.PrintMsg that prints to the trailer,
    ## NOT print.  Since I do NOT have access to the FD opened 
    ## by opusutil, I created a new method there instead.
    opusutil.PrintStr( line )

opusutil.PrintMsg('I', endgMsg )
## No need to attempt to capture errors, because failure to close 
## is not an error
opusutil.CloseTrl()

sys.exit(ALL_IS_WELL)
